"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var view_1 = require("tns-core-modules/ui/core/view");
var stack_layout_1 = require("tns-core-modules/ui/layouts/stack-layout");
var observable_array_1 = require("tns-core-modules/data/observable-array");
var types = require("tns-core-modules/utils/types");
var platform_1 = require("tns-core-modules/platform/platform");
var pager_common_1 = require("./pager.common");
__export(require("./pager.common"));
function notifyForItemAtIndex(owner, nativeView, view, eventName, index) {
    var args = {
        eventName: eventName,
        object: owner,
        index: index,
        view: view,
        ios: undefined,
        android: nativeView
    };
    owner.notify(args);
    return args;
}
var pager_common_2 = require("./pager.common");
exports.Transformer = pager_common_2.Transformer;
var PLACEHOLDER = 'PLACEHOLDER';
var Pager = (function (_super) {
    __extends(Pager, _super);
    function Pager() {
        var _this = _super.call(this) || this;
        _this._oldDisableAnimation = false;
        _this._realizedItems = new Map();
        _this._realizedTemplates = new Map();
        _this.lastEvent = 0;
        _this._lastSpacing = 0;
        _this._lastPeaking = 0;
        _this._selectedIndexBeforeLoad = 0;
        _this._observableArrayHandler = function (args) {
            if (_this.indicatorView) {
                _this.indicatorView.setCount(_this._childrenCount);
            }
            if (_this.pagerAdapter) {
                switch (args.action) {
                    case observable_array_1.ChangeType.Add:
                        _this.pagerAdapter.notifyItemRangeInserted(args.index, args.addedCount);
                        break;
                    case observable_array_1.ChangeType.Delete:
                        _this.pagerAdapter.notifyItemRangeRemoved(args.index, args.removed.length);
                        break;
                    case observable_array_1.ChangeType.Splice:
                        if (args.removed.length > 0) {
                            _this.pagerAdapter.notifyItemRangeRemoved(args.index, args.removed.length);
                        }
                        if (args.addedCount > 0) {
                            _this.pagerAdapter.notifyItemRangeInserted(args.index, args.addedCount);
                        }
                        break;
                    case observable_array_1.ChangeType.Update:
                        _this.pagerAdapter.notifyItemChanged(args.index);
                        break;
                    default:
                        break;
                }
            }
        };
        _this._horizontalOffset = 0;
        _this._verticalOffset = 0;
        _this._childrenViews = new Map();
        _this._transformers = [];
        return _this;
    }
    Pager.prototype.itemTemplateUpdated = function (oldData, newData) {
    };
    Object.defineProperty(Pager.prototype, "views", {
        get: function () {
            return this._views;
        },
        set: function (value) {
            this._views = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pager.prototype, "android", {
        get: function () {
            return this.nativeViewProtected;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pager.prototype, "pager", {
        get: function () {
            return this._pager;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pager.prototype, "indicatorView", {
        get: function () {
            return this._indicatorView;
        },
        enumerable: true,
        configurable: true
    });
    Pager.prototype.createNativeView = function () {
        var that = new WeakRef(this);
        var nativeView = new android.widget.RelativeLayout(this._context);
        this._pager = new androidx.viewpager2.widget.ViewPager2(this._context);
        if (this.orientation === 'vertical') {
            this.pager.setOrientation(androidx.viewpager2.widget.ViewPager2.ORIENTATION_VERTICAL);
        }
        else {
            this.pager.setOrientation(androidx.viewpager2.widget.ViewPager2
                .ORIENTATION_HORIZONTAL);
        }
        initPagerChangeCallback();
        this._pageListener = new PageChangeCallback(that);
        initPagerRecyclerAdapter();
        this._pagerAdapter = new PagerRecyclerAdapter(new WeakRef(this));
        this.compositeTransformer = new androidx.viewpager2.widget.CompositePageTransformer();
        if (this.pagesCount > 0) {
            this.pager.setOffscreenPageLimit(this.pagesCount);
        }
        else {
            this.pager.setOffscreenPageLimit(3);
        }
        this.pager.setUserInputEnabled(!this.disableSwipe);
        this.on(view_1.View.layoutChangedEvent, this.onLayoutChange, this);
        nativeView.addView(this.pager);
        this._indicatorView = new com.rd.PageIndicatorView2(this._context);
        var params = new android.widget.RelativeLayout.LayoutParams(android.widget.RelativeLayout.LayoutParams.WRAP_CONTENT, android.widget.RelativeLayout.LayoutParams.WRAP_CONTENT);
        params.addRule(android.widget.RelativeLayout.ALIGN_PARENT_BOTTOM);
        params.addRule(android.widget.RelativeLayout.CENTER_HORIZONTAL);
        params.setMargins(0, 0, 0, 10 * platform_1.screen.mainScreen.scale);
        this._indicatorView.setLayoutParams(params);
        this._indicatorView.setViewPager(this.pager);
        if (this.showIndicator) {
            nativeView.addView(this._indicatorView);
        }
        return nativeView;
    };
    Pager.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        this._oldDisableAnimation = this.disableAnimation;
        this.disableAnimation = true;
        this.pager.registerOnPageChangeCallback(this._pageListener);
        this.pager.setAdapter(this._pagerAdapter);
        if (this._androidViewId < 0) {
            this._androidViewId = android.view.View.generateViewId();
        }
        if (this.pagesCount > 0) {
            this.pager.setOffscreenPageLimit(this.pagesCount);
        }
        this._setIndicator(this.indicator);
        this.nativeView.setId(this._androidViewId);
        this._setPeaking(this.peaking);
        this._setSpacing(this.spacing);
        this._setTransformers(this.transformers ? this.transformers : '');
    };
    Pager.prototype.onLayoutChange = function (args) {
        this._setSpacing(args.object.spacing);
        this._setPeaking(args.object.peaking);
        this._setTransformers(this.transformers ? this.transformers : '');
        this._updateScrollPosition();
        this.disableAnimation = this._oldDisableAnimation;
    };
    Pager.prototype._setSpacing = function (value) {
        var size = this.convertToSize(value);
        var newSpacing = size !== this._lastSpacing;
        if (newSpacing) {
            if (this.marginTransformer) {
                this.compositeTransformer.removeTransformer(this.marginTransformer);
            }
            this.marginTransformer = new androidx.viewpager2.widget.MarginPageTransformer(size);
            this.compositeTransformer.addTransformer(this.marginTransformer);
            this._lastSpacing = size;
        }
    };
    Pager.prototype._setPeaking = function (value) {
        var size = this.convertToSize(value);
        var newPeaking = size !== this._lastPeaking;
        if (newPeaking) {
            this.pager.setClipToPadding(false);
            var left = this.orientation === 'horizontal' ? size : 0;
            var top_1 = this.orientation === 'horizontal' ? 0 : size;
            this.pager.setPadding(left, top_1, left, top_1);
            this.pager.setClipChildren(false);
            this._lastPeaking = size;
        }
    };
    Pager.prototype[pager_common_1.spacingProperty.setNative] = function (value) {
        this._setSpacing(value);
    };
    Pager.prototype[pager_common_1.peakingProperty.setNative] = function (value) {
        this._setPeaking(value);
    };
    Pager.prototype[pager_common_1.indicatorProperty.setNative] = function (value) {
        this._setIndicator(value);
    };
    Pager.prototype._setIndicator = function (value) {
        var AnimationType = com.rd.animation.type.AnimationType;
        switch (value) {
            case pager_common_1.Indicator.None:
                this.indicatorView.setAnimationType(AnimationType.NONE);
                break;
            case pager_common_1.Indicator.Worm:
                this.indicatorView.setAnimationType(AnimationType.WORM);
                break;
            case pager_common_1.Indicator.Fill:
                this.indicatorView.setAnimationType(AnimationType.FILL);
                break;
            case pager_common_1.Indicator.Swap:
                this.indicatorView.setAnimationType(AnimationType.SWAP);
                break;
            case pager_common_1.Indicator.THIN_WORM:
                this.indicatorView.setAnimationType(AnimationType.THIN_WORM);
                break;
            default:
                break;
        }
    };
    Pager.prototype._setTransformers = function (transformers) {
        var _this = this;
        if (!types.isString(transformers)) {
            return;
        }
        var transformsArray = transformers.split(' ');
        this._transformers.forEach(function (transformer) {
            _this.compositeTransformer.removeTransformer(transformer);
        });
        for (var _i = 0, transformsArray_1 = transformsArray; _i < transformsArray_1.length; _i++) {
            var transformer = transformsArray_1[_i];
            if (transformer === pager_common_1.Transformer.SCALE) {
                initZoomOutPageTransformer();
                var nativeTransformer = new ZoomOutPageTransformer();
                nativeTransformer.owner = new WeakRef(this);
                this._transformers.push(nativeTransformer);
                this.compositeTransformer.addTransformer(nativeTransformer);
            }
        }
        if (transformsArray.length === 0) {
            this._transformers.forEach(function (transformer) {
                _this.compositeTransformer.removeTransformer(transformer);
            });
        }
        this.pager.setPageTransformer(this.compositeTransformer);
    };
    Pager.prototype.disposeNativeView = function () {
        this.off(view_1.View.layoutChangedEvent, this.onLayoutChange, this);
        this._childrenViews.clear();
        this._realizedItems.clear();
        this._realizedTemplates.clear();
        this._pageListener = null;
        this._pagerAdapter = null;
        this._transformers = [];
        if (this._observableArrayInstance) {
            this._observableArrayInstance.off(observable_array_1.ObservableArray.changeEvent, this._observableArrayHandler);
            this._observableArrayInstance = null;
        }
        _super.prototype.disposeNativeView.call(this);
    };
    Object.defineProperty(Pager.prototype, "disableAnimation", {
        get: function () {
            return this._disableAnimation;
        },
        set: function (value) {
            this._disableAnimation = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pager.prototype, "pagerAdapter", {
        get: function () {
            return this._pagerAdapter;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pager.prototype, "_childrenCount", {
        get: function () {
            return this.items
                ? this.items.length
                : this._childrenViews
                    ? this._childrenViews.size
                    : 0;
        },
        enumerable: true,
        configurable: true
    });
    Pager.prototype[pager_common_1.disableSwipeProperty.setNative] = function (value) {
        if (this.pager) {
            this.pager.setUserInputEnabled(!value);
        }
    };
    Pager.prototype[pager_common_1.itemsProperty.getDefault] = function () {
        return null;
    };
    Pager.prototype[pager_common_1.itemsProperty.setNative] = function (value) {
        if (value && value.length) {
            this.indicatorView.setCount(this._childrenCount);
        }
        if (this._observableArrayInstance) {
            this._observableArrayInstance.off(observable_array_1.ObservableArray.changeEvent, this._observableArrayHandler);
            this._observableArrayInstance = null;
        }
        if (value) {
            if (value instanceof observable_array_1.ObservableArray) {
                var adapter = this.pagerAdapter;
                if (!adapter)
                    return;
                pager_common_1.selectedIndexProperty.coerce(this);
                this._observableArrayInstance = value;
                this._observableArrayInstance.on(observable_array_1.ObservableArray.changeEvent, this._observableArrayHandler);
            }
            else {
                this.refresh();
                pager_common_1.selectedIndexProperty.coerce(this);
            }
        }
    };
    Pager.prototype._updateScrollPosition = function () {
        if (this.pager.getCurrentItem() !== this.selectedIndex) {
            this.pager.setCurrentItem(this.selectedIndex, false);
            this._indicatorView.setSelection(this.selectedIndex);
        }
    };
    Pager.prototype.onLoaded = function () {
        var _this = this;
        _super.prototype.onLoaded.call(this);
        if (!this.items && this._childrenCount > 0) {
            initStaticPagerStateAdapter();
            this._pagerAdapter = new StaticPagerStateAdapter(new WeakRef(this));
            this.pager.setAdapter(this._pagerAdapter);
            pager_common_1.selectedIndexProperty.coerce(this);
            setTimeout(function () {
                _this.pager.setCurrentItem(_this.selectedIndex, false);
            }, 0);
        }
    };
    Pager.prototype[pager_common_1.selectedIndexProperty.setNative] = function (value) {
        if (this.isLoaded && this.isLayoutValid && this.pager) {
            if (this.pager.getCurrentItem() !== value) {
                this.pager.setCurrentItem(value, !this.disableAnimation);
            }
        }
    };
    Pager.prototype.scrollToIndexAnimated = function (index, animate) {
        if (this.pager) {
            this.pager.setCurrentItem(index, animate);
        }
    };
    Pager.prototype._onItemsChanged = function (oldValue, newValue) {
    };
    Pager.prototype.refresh = function () {
        if (this.pager && this._pagerAdapter) {
            this.pager.requestLayout();
            this.pager.getAdapter()
                .notifyDataSetChanged();
        }
    };
    Pager.prototype.updatePagesCount = function (value) {
        if (this.pager) {
            this._pagerAdapter.notifyDataSetChanged();
            this.pager.setOffscreenPageLimit(value);
        }
    };
    Pager.prototype.onUnloaded = function () {
        _super.prototype.onUnloaded.call(this);
    };
    Pager.prototype.eachChildView = function (callback) {
        if (this._realizedItems && this._realizedItems.size > 0) {
            this._realizedItems.forEach(function (view, key) {
                callback(view);
            });
        }
    };
    Pager.prototype.updateAdapter = function () {
        this._pagerAdapter.notifyDataSetChanged();
    };
    Pager.prototype._selectedIndexUpdatedFromNative = function (newIndex) {
    };
    Pager.prototype[pager_common_1.itemTemplatesProperty.getDefault] = function () {
        return null;
    };
    Pager.prototype[pager_common_1.itemTemplatesProperty.setNative] = function (value) {
        this._itemTemplatesInternal = new Array(this._defaultTemplate);
        if (value) {
            this._itemTemplatesInternal = this._itemTemplatesInternal.concat(value);
        }
        this._pagerAdapter = new PagerRecyclerAdapter();
        this._pagerAdapter.owner = new WeakRef(this);
        this.pager.setAdapter(this._pagerAdapter);
        this.refresh();
    };
    Pager.prototype[pager_common_1.showIndicatorProperty.setNative] = function (value) {
        var hasParent = this.indicatorView.getParent();
        if (value) {
            if (!hasParent) {
                this.nativeView.addView(this.indicatorView);
            }
        }
        else {
            if (hasParent) {
                this.nativeView.removeView(this.indicatorView);
            }
        }
    };
    Pager.prototype._addChildFromBuilder = function (name, value) {
        if (value instanceof pager_common_1.PagerItem) {
            if (!value.parent && value.parent !== this) {
                this._childrenViews.set(this._childrenViews.size, value);
            }
        }
    };
    Pager.prototype[pager_common_1.orientationProperty.setNative] = function (value) {
        if (value === 'vertical') {
            this.nativeViewProtected.setOrientation(androidx.viewpager2.widget.ViewPager2.ORIENTATION_VERTICAL);
        }
        else {
            this.nativeViewProtected.setOrientation(androidx.viewpager2.widget.ViewPager2
                .ORIENTATION_HORIZONTAL);
        }
    };
    Object.defineProperty(Pager.prototype, "horizontalOffset", {
        get: function () {
            return this._horizontalOffset / view_1.layout.getDisplayDensity();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pager.prototype, "verticalOffset", {
        get: function () {
            return this._verticalOffset / view_1.layout.getDisplayDensity();
        },
        enumerable: true,
        configurable: true
    });
    return Pager;
}(pager_common_1.PagerBase));
exports.Pager = Pager;
exports.pagesCountProperty = new view_1.Property({
    name: 'pagesCount',
    defaultValue: 0,
    valueConverter: function (v) { return parseInt(v); },
    valueChanged: function (pager, oldValue, newValue) {
        pager.updatePagesCount(pager.pagesCount);
    }
});
exports.pagesCountProperty.register(Pager);
var PageChangeCallback;
function initPagerChangeCallback() {
    if (PageChangeCallback) {
        return PageChangeCallback;
    }
    var PageChangeCallbackImpl = (function (_super) {
        __extends(PageChangeCallbackImpl, _super);
        function PageChangeCallbackImpl(owner) {
            var _this = _super.call(this) || this;
            _this.owner = owner;
            return global.__native(_this);
        }
        PageChangeCallbackImpl.prototype.onPageSelected = function (position) {
            var owner = this.owner && this.owner.get();
            if (owner) {
                if (owner.isLayoutValid && owner.selectedIndex !== position) {
                    owner.indicatorView.setSelection(position);
                    pager_common_1.selectedIndexProperty.nativeValueChange(owner, position);
                }
                owner.notify({
                    eventName: Pager.swipeEvent,
                    object: owner
                });
            }
        };
        PageChangeCallbackImpl.prototype.onPageScrolled = function (position, positionOffset, positionOffsetPixels) {
            var offset = position * positionOffsetPixels;
            var owner = this.owner && this.owner.get();
            if (owner) {
                if (owner.orientation === 'vertical') {
                    owner._horizontalOffset = 0;
                    owner._verticalOffset = offset;
                }
                else if (owner.orientation === 'horizontal') {
                    owner._horizontalOffset = offset;
                    owner._verticalOffset = 0;
                }
                owner.notify({
                    eventName: Pager.scrollEvent,
                    object: owner,
                    selectedIndex: position,
                    scrollX: owner.horizontalOffset,
                    scrollY: owner.verticalOffset
                });
                if (owner.items && position === (owner.items.length - 1) - owner.loadMoreCount) {
                    owner.notify({ eventName: pager_common_1.LOADMOREITEMS, object: owner });
                }
            }
        };
        PageChangeCallbackImpl.prototype.onPageScrollStateChanged = function (state) {
            var owner = this.owner && this.owner.get();
            if (owner) {
                if (owner.lastEvent === 0 && state === 1) {
                    owner.notify({
                        eventName: Pager.swipeStartEvent,
                        object: owner
                    });
                    owner.lastEvent = 1;
                }
                else if (owner.lastEvent === 1 && state === 1) {
                    owner.notify({
                        eventName: Pager.swipeOverEvent,
                        object: owner
                    });
                    owner.lastEvent = 1;
                }
                else if (owner.lastEvent === 1 && state === 2) {
                    owner.notify({
                        eventName: Pager.swipeEndEvent,
                        object: owner
                    });
                    owner.lastEvent = 2;
                }
                else {
                    owner.lastEvent = 0;
                }
            }
        };
        return PageChangeCallbackImpl;
    }(androidx.viewpager2.widget.ViewPager2.OnPageChangeCallback));
    PageChangeCallback = PageChangeCallbackImpl;
}
var PagerRecyclerAdapter;
function initPagerRecyclerAdapter() {
    if (PagerRecyclerAdapter) {
        return;
    }
    var PagerRecyclerAdapterImpl = (function (_super) {
        __extends(PagerRecyclerAdapterImpl, _super);
        function PagerRecyclerAdapterImpl(owner) {
            var _this = _super.call(this) || this;
            _this.owner = owner;
            return global.__native(_this);
        }
        PagerRecyclerAdapterImpl.prototype.onCreateViewHolder = function (param0, type) {
            var owner = this.owner ? this.owner.get() : null;
            if (!owner) {
                return null;
            }
            var template = owner._itemTemplatesInternal[type];
            var view = template.createView();
            var sp = new stack_layout_1.StackLayout();
            if (view) {
                sp.addChild(view);
            }
            else {
                sp[PLACEHOLDER] = true;
            }
            owner._addView(sp);
            sp.nativeView.setLayoutParams(new android.view.ViewGroup.LayoutParams(android.view.ViewGroup.LayoutParams.MATCH_PARENT, android.view.ViewGroup.LayoutParams.MATCH_PARENT));
            owner._realizedItems.set(sp.nativeView, sp);
            initPagerViewHolder();
            return new PagerViewHolder(new WeakRef(sp), new WeakRef(owner));
        };
        PagerRecyclerAdapterImpl.prototype.onBindViewHolder = function (holder, index) {
            var owner = this.owner ? this.owner.get() : null;
            if (owner) {
                var args = {
                    eventName: pager_common_1.ITEMLOADING,
                    object: owner,
                    android: holder,
                    ios: undefined,
                    index: index,
                    view: holder.view[PLACEHOLDER] ? null : holder.view
                };
                owner.notify(args);
                if (holder.view[PLACEHOLDER]) {
                    if (args.view) {
                        holder.view.addChild(args.view);
                    }
                    else {
                        holder.view.addChild(owner._getDefaultItemContent(index));
                    }
                    holder.view[PLACEHOLDER] = false;
                }
                owner._prepareItem(holder.view, index);
            }
        };
        PagerRecyclerAdapterImpl.prototype.getItemId = function (i) {
            var owner = this.owner ? this.owner.get() : null;
            var id = i;
            if (owner && owner.items) {
                var item = owner.items.getItem ? owner.items.getItem(i) : owner.items[i];
                if (item) {
                    id = owner.itemIdGenerator(item, i, owner.items);
                }
            }
            return long(id);
        };
        PagerRecyclerAdapterImpl.prototype.getItemCount = function () {
            var owner = this.owner ? this.owner.get() : null;
            return owner && owner.items && owner.items.length
                ? owner.items.length
                : 0;
        };
        PagerRecyclerAdapterImpl.prototype.getItemViewType = function (index) {
            var owner = this.owner ? this.owner.get() : null;
            if (owner) {
                var template = owner._getItemTemplate(index);
                return owner._itemTemplatesInternal.indexOf(template);
            }
            return 0;
        };
        PagerRecyclerAdapterImpl.prototype.hasStableIds = function () {
            return true;
        };
        return PagerRecyclerAdapterImpl;
    }(androidx.recyclerview.widget.RecyclerView.Adapter));
    PagerRecyclerAdapter = PagerRecyclerAdapterImpl;
}
var StaticPagerStateAdapter;
function initStaticPagerStateAdapter() {
    if (StaticPagerStateAdapter) {
        return;
    }
    var StaticPagerStateAdapterImpl = (function (_super) {
        __extends(StaticPagerStateAdapterImpl, _super);
        function StaticPagerStateAdapterImpl(owner) {
            var _this = _super.call(this) || this;
            _this.owner = owner;
            return global.__native(_this);
        }
        StaticPagerStateAdapterImpl.prototype.onCreateViewHolder = function (param0, type) {
            var owner = this.owner ? this.owner.get() : null;
            if (!owner) {
                return null;
            }
            var view = owner._childrenViews.get(type);
            var sp = new stack_layout_1.StackLayout();
            if (view && !view.parent) {
                sp.addChild(view);
            }
            else {
                sp[PLACEHOLDER] = true;
            }
            owner._addView(sp);
            sp.nativeView.setLayoutParams(new android.view.ViewGroup.LayoutParams(android.view.ViewGroup.LayoutParams.MATCH_PARENT, android.view.ViewGroup.LayoutParams.MATCH_PARENT));
            initPagerViewHolder();
            return new PagerViewHolder(new WeakRef(sp), new WeakRef(owner));
        };
        StaticPagerStateAdapterImpl.prototype.onBindViewHolder = function (holder, index) {
            var owner = this.owner ? this.owner.get() : null;
            if (owner) {
                var args = {
                    eventName: pager_common_1.ITEMLOADING,
                    object: owner,
                    android: holder,
                    ios: undefined,
                    index: index,
                    view: holder.view[PLACEHOLDER] ? null : holder.view
                };
                owner.notify(args);
                if (holder.view[PLACEHOLDER]) {
                    if (args.view) {
                        holder.view.addChild(args.view);
                    }
                    holder.view[PLACEHOLDER] = false;
                }
            }
        };
        StaticPagerStateAdapterImpl.prototype.hasStableIds = function () {
            return true;
        };
        StaticPagerStateAdapterImpl.prototype.getItem = function (i) {
            var owner = this.owner ? this.owner.get() : null;
            if (owner) {
                if (owner._childrenViews) {
                    return owner._childrenViews.get(i);
                }
            }
            return null;
        };
        StaticPagerStateAdapterImpl.prototype.getItemId = function (i) {
            var owner = this.owner ? this.owner.get() : null;
            var id = i;
            if (owner) {
                var item = this.getItem(i);
                if (item) {
                    id = owner.itemIdGenerator(item, i, Array.from(owner._childrenViews));
                }
            }
            return long(id);
        };
        StaticPagerStateAdapterImpl.prototype.getItemCount = function () {
            var owner = this.owner ? this.owner.get() : null;
            return owner && owner._childrenViews
                ? owner._childrenViews.size
                : 0;
        };
        StaticPagerStateAdapterImpl.prototype.getItemViewType = function (index) {
            return index;
        };
        return StaticPagerStateAdapterImpl;
    }(androidx.recyclerview.widget.RecyclerView.Adapter));
    StaticPagerStateAdapter = StaticPagerStateAdapterImpl;
}
var PagerViewHolder;
function initPagerViewHolder() {
    if (PagerViewHolder) {
        return;
    }
    var PagerViewHolderImpl = (function (_super) {
        __extends(PagerViewHolderImpl, _super);
        function PagerViewHolderImpl(owner, pager) {
            var _this = _super.call(this, owner.get().nativeViewProtected) || this;
            _this.owner = owner;
            _this.pager = pager;
            return global.__native(_this);
        }
        Object.defineProperty(PagerViewHolderImpl.prototype, "view", {
            get: function () {
                return this.owner ? this.owner.get() : null;
            },
            enumerable: true,
            configurable: true
        });
        return PagerViewHolderImpl;
    }(androidx.recyclerview.widget.RecyclerView.ViewHolder));
    PagerViewHolder = PagerViewHolderImpl;
}
var ZoomOutPageTransformer;
function initZoomOutPageTransformer() {
    if (ZoomOutPageTransformer) {
        return;
    }
    var ZoomOutPageTransformerImpl = (function (_super) {
        __extends(ZoomOutPageTransformerImpl, _super);
        function ZoomOutPageTransformerImpl() {
            var _this = _super.call(this) || this;
            return global.__native(_this);
        }
        ZoomOutPageTransformerImpl.prototype.transformPage = function (view, position) {
            var MIN_SCALE = 0.85;
            if (position <= 1 || position >= -1) {
                var scale = Math.max(MIN_SCALE, 1 - Math.abs(position));
                view.setScaleX(scale);
                view.setScaleY(scale);
            }
            else {
                view.setScaleX(1);
                view.setScaleY(1);
            }
        };
        ZoomOutPageTransformerImpl = __decorate([
            Interfaces([androidx.viewpager2.widget.ViewPager2.PageTransformer]),
            __metadata("design:paramtypes", [])
        ], ZoomOutPageTransformerImpl);
        return ZoomOutPageTransformerImpl;
    }(java.lang.Object));
    ZoomOutPageTransformer = ZoomOutPageTransformerImpl;
}
var ZoomInPageTransformer;
function initZoomInPageTransformer() {
    if (ZoomInPageTransformer) {
        return;
    }
    var ZoomInPageTransformerImpl = (function (_super) {
        __extends(ZoomInPageTransformerImpl, _super);
        function ZoomInPageTransformerImpl() {
            var _this = _super.call(this) || this;
            return global.__native(_this);
        }
        ZoomInPageTransformerImpl.prototype.transformPage = function (view, position) {
            var scale = position < 0 ? position + 1.0 : Math.abs(1.0 - position);
            view.setScaleX(scale);
            view.setScaleY(scale);
            view.setPivotX(view.getWidth() * 0.5);
            view.setPivotY(view.getHeight() * 0.5);
            view.setAlpha(view < -1.0 || position > 1.0 ? 0.0 : 1.0 - (scale - 1.0));
        };
        ZoomInPageTransformerImpl = __decorate([
            Interfaces([androidx.viewpager2.widget.ViewPager2.PageTransformer]),
            __metadata("design:paramtypes", [])
        ], ZoomInPageTransformerImpl);
        return ZoomInPageTransformerImpl;
    }(java.lang.Object));
    ZoomInPageTransformer = ZoomInPageTransformerImpl;
}
//# sourceMappingURL=pager.android.js.map