"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var view_1 = require("tns-core-modules/ui/core/view");
var stack_layout_1 = require("tns-core-modules/ui/layouts/stack-layout");
var proxy_view_container_1 = require("tns-core-modules/ui/proxy-view-container");
var common = require("./pager.common");
var pager_common_1 = require("./pager.common");
var profiling_1 = require("tns-core-modules/profiling");
var observable_array_1 = require("tns-core-modules/data/observable-array/observable-array");
var pager_common_2 = require("./pager.common");
exports.Transformer = pager_common_2.Transformer;
function notifyForItemAtIndex(owner, nativeView, view, eventName, index) {
    var args = {
        eventName: eventName,
        object: owner,
        index: index,
        view: view,
        ios: nativeView,
        android: undefined
    };
    owner.notify(args);
    return args;
}
var main_queue = dispatch_get_current_queue();
__export(require("./pager.common"));
var Pager = (function (_super) {
    __extends(Pager, _super);
    function Pager() {
        var _this = _super.call(this) || this;
        _this.lastEvent = 0;
        _this._disableSwipe = false;
        _this._disableAnimation = false;
        _this._initialLoad = false;
        _this._preparingCell = false;
        _this._isDirty = false;
        _this._isRefreshing = false;
        _this._observableArrayHandler = function (args) {
            if (!_this.pager) {
                return;
            }
            if (_this.indicatorView && _this._observableArrayInstance && _this._observableArrayInstance.length) {
                _this.indicatorView.numberOfPages = _this._observableArrayInstance.length;
            }
            dispatch_async(main_queue, function () {
                var collectionView = _this.pager;
                collectionView.performBatchUpdatesCompletion(function () {
                    _this._isRefreshing = true;
                    var array = [];
                    var count = 0;
                    switch (args.action) {
                        case observable_array_1.ChangeType.Add:
                            count = args.index + args.addedCount;
                            for (var i = args.index; i < count; i++) {
                                array.push(NSIndexPath.indexPathForRowInSection(i, 0));
                            }
                            collectionView.insertItemsAtIndexPaths(array);
                            break;
                        case observable_array_1.ChangeType.Delete:
                            args.removed.forEach(function (item) {
                                var index = _this.items.indexOf(item);
                                if (index > -1) {
                                    array.push(NSIndexPath.indexPathForItemInSection(index, 0));
                                }
                            });
                            if (array.length > 0) {
                                collectionView.deleteItemsAtIndexPaths(array);
                            }
                            break;
                        case observable_array_1.ChangeType.Splice:
                            if (args.removed && args.removed.length > 0) {
                                count = args.index + args.removed.length;
                                for (var i = args.index; i < count; i++) {
                                    array.push(NSIndexPath.indexPathForRowInSection(i, 0));
                                }
                                if (array.length > 0) {
                                    collectionView.deleteItemsAtIndexPaths(array);
                                }
                            }
                            if (args.addedCount > 0) {
                                var addedArray = [];
                                count = args.index + args.addedCount;
                                for (var i = args.index; i < count; i++) {
                                    addedArray.push(NSIndexPath.indexPathForRowInSection(i, 0));
                                }
                                collectionView.insertItemsAtIndexPaths(addedArray);
                            }
                            break;
                        case observable_array_1.ChangeType.Update:
                            collectionView.reloadItemsAtIndexPaths([NSIndexPath.indexPathForRowInSection(args.index, 0)]);
                            break;
                        default:
                            break;
                    }
                }, null);
            });
        };
        _this._map = new Map();
        _this._childrenViews = new Map();
        return _this;
    }
    Object.defineProperty(Pager.prototype, "pager", {
        get: function () {
            return this._pager;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pager.prototype, "indicatorView", {
        get: function () {
            return this._indicatorView;
        },
        enumerable: true,
        configurable: true
    });
    Pager.prototype.createNativeView = function () {
        var nativeView = UIView.new();
        this._layout = UICollectionViewFlowLinearLayoutImpl.initWithOwner(new WeakRef(this));
        this._layout.scrollDirection = 1;
        this._layout.minimumInteritemSpacing = 0;
        this._pager = UICollectionView.alloc().initWithFrameCollectionViewLayout(CGRectZero, this._layout);
        this.pager.showsHorizontalScrollIndicator = false;
        this.pager.showsVerticalScrollIndicator = false;
        this.pager.decelerationRate = UIScrollViewDecelerationRateFast;
        nativeView.addSubview(this.pager);
        return nativeView;
    };
    Pager.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.pager;
        nativeView.registerClassForCellWithReuseIdentifier(PagerCell.class(), this._defaultTemplate.key);
        nativeView.backgroundColor = UIColor.clearColor;
        nativeView.autoresizesSubviews = false;
        nativeView.autoresizingMask = 0;
        nativeView.dataSource = this._dataSource = UICollectionViewDataSourceImpl.initWithOwner(new WeakRef(this));
        nativeView.scrollEnabled = !(String(this.disableSwipe) === 'true');
        if (this.orientation === 'vertical') {
            this._layout.scrollDirection = 0;
            nativeView.alwaysBounceVertical = true;
            nativeView.alwaysBounceHorizontal = false;
        }
        else {
            this._layout.scrollDirection = 1;
            nativeView.alwaysBounceHorizontal = true;
            nativeView.alwaysBounceVertical = false;
        }
        this._setIndicator(this.indicator);
        this._delegate = UICollectionDelegateImpl.initWithOwner(new WeakRef(this));
        this._setNativeClipToBounds();
    };
    Object.defineProperty(Pager.prototype, "ios", {
        get: function () {
            return this.nativeView;
        },
        enumerable: true,
        configurable: true
    });
    Pager.prototype._setIndicator = function (value) {
        if (this._indicatorView) {
            this._indicatorView.removeFromSuperview();
        }
        switch (value) {
            case pager_common_1.Indicator.None:
                this._indicatorView = CHIPageControlAji.new();
                break;
            case pager_common_1.Indicator.Worm:
                this._indicatorView = CHIPageControlAleppo.new();
                break;
            case pager_common_1.Indicator.Fill:
                this._indicatorView = CHIPageControlChimayo.new();
                break;
            case pager_common_1.Indicator.Swap:
                this._indicatorView = CHIPageControlPuya.new();
                break;
            case pager_common_1.Indicator.THIN_WORM:
                this._indicatorView = CHIPageControlJalapeno.new();
                break;
            case pager_common_1.Indicator.Flat:
                this._indicatorView = CHIPageControlJaloro.new();
                break;
            default:
                break;
        }
        this._indicatorView.tintColor = UIColor.whiteColor;
        this._indicatorView.currentPageTintColor = UIColor.whiteColor;
    };
    Object.defineProperty(Pager.prototype, "_childrenCount", {
        get: function () {
            return this.items ? this.items.length : this._childrenViews ? this._childrenViews.size : 0;
        },
        enumerable: true,
        configurable: true
    });
    Pager.prototype.itemTemplateUpdated = function (oldData, newData) {
    };
    Pager.prototype._setNativeClipToBounds = function () {
        this.pager.clipsToBounds = true;
    };
    Pager.prototype[pager_common_1.orientationProperty.setNative] = function (value) {
        if (value === 'horizontal') {
            this._layout.scrollDirection = 1;
        }
        else {
            this._layout.scrollDirection = 0;
        }
    };
    Pager.prototype.eachChildView = function (callback) {
        this._map.forEach(function (view, key) {
            callback(view);
        });
    };
    Pager.prototype._updateScrollPosition = function () {
        var view = this.pager;
        if (!view || view.contentSize.width === 0) {
            return;
        }
        if (this.indicatorView) {
            this.indicatorView.progress = this.selectedIndex;
        }
        this._scrollToIndexAnimated(this.selectedIndex, false);
    };
    Pager.prototype[pager_common_1.selectedIndexProperty.setNative] = function (value) {
        if (this.isLoaded) {
            this.scrollToIndexAnimated(value, true);
        }
    };
    Pager.prototype[pager_common_1.itemTemplatesProperty.getDefault] = function () {
        return null;
    };
    Pager.prototype[pager_common_1.itemTemplatesProperty.setNative] = function (value) {
        this._itemTemplatesInternal = new Array(this._defaultTemplate);
        if (value) {
            for (var i = 0, length_1 = value.length; i < length_1; i++) {
                this.pager.registerClassForCellWithReuseIdentifier(PagerCell.class(), value[i].key);
            }
            this._itemTemplatesInternal = this._itemTemplatesInternal.concat(value);
        }
    };
    Pager.prototype[pager_common_1.itemsProperty.setNative] = function (value) {
        if (this.indicatorView && value && value.length) {
            this.indicatorView.numberOfPages = value.length;
        }
        if (this._observableArrayInstance) {
            this._observableArrayInstance.off(observable_array_1.ObservableArray.changeEvent, this._observableArrayHandler);
            this._observableArrayInstance = null;
        }
        if (value instanceof observable_array_1.ObservableArray) {
            this._observableArrayInstance = value;
            this._observableArrayInstance.on(observable_array_1.ObservableArray.changeEvent, this._observableArrayHandler);
        }
        else {
            this.refresh();
        }
        pager_common_1.selectedIndexProperty.coerce(this);
    };
    Pager.prototype[pager_common_1.showIndicatorProperty.setNative] = function (value) {
        if (!this.indicatorView) {
            this._setIndicator(this.indicatorView);
        }
        if (!this.nativeView) {
            return;
        }
        this.indicatorView.center = CGPointMake(this.nativeView.center.x, this.nativeView.bounds.size.height - this.indicatorView.intrinsicContentSize.height);
        var hasParent = this.indicatorView.superview;
        if (value) {
            if (!hasParent) {
                this.nativeView.addSubview(this.indicatorView);
            }
        }
        else {
            if (hasParent) {
                this.nativeView.removeFromSuperview();
            }
        }
    };
    Pager.prototype._onItemsChanged = function (oldValue, newValue) {
    };
    Pager.prototype._scrollToIndexAnimated = function (index, animate) {
        var _this = this;
        if (!this._pager)
            return;
        if (this._childrenCount === 0) {
            return;
        }
        var maxMinIndex = -1;
        var max = this._childrenCount - 1;
        if (index < 0) {
            maxMinIndex = 0;
        }
        else if (index > max) {
            maxMinIndex = max;
        }
        else {
            maxMinIndex = index;
        }
        if (maxMinIndex === -1) {
            maxMinIndex = 0;
        }
        dispatch_async(main_queue, function () {
            _this.pager.scrollToItemAtIndexPathAtScrollPositionAnimated(NSIndexPath.indexPathForItemInSection(maxMinIndex, 0), _this.orientation === 'vertical' ? 2 : 16, !!animate);
        });
    };
    Pager.prototype.scrollToIndexAnimated = function (index, animate) {
        this._scrollToIndexAnimated(index, animate);
    };
    Pager.prototype._reset = function () {
        if (!this.pager) {
            return;
        }
        this.pager.reloadData();
        this.pager.collectionViewLayout.invalidateLayout();
        this._updateScrollPosition();
    };
    Pager.prototype._refresh = function () {
        var _this = this;
        if (!this.pager) {
            return;
        }
        if (this.items instanceof observable_array_1.ObservableArray) {
            this.pager.performBatchUpdatesCompletion(function () {
                _this._reset();
            }, null);
        }
        else {
            this._reset();
        }
    };
    Pager.prototype.refresh = function () {
        var _this = this;
        dispatch_async(main_queue, function () {
            _this._refresh();
        });
    };
    Pager.prototype.onLoaded = function () {
        _super.prototype.onLoaded.call(this);
        if (this.showIndicator && this.indicatorView) {
            this.nativeView.addSubview(this.indicatorView);
        }
        if (!this._isDirty) {
            this.refresh();
            this._isDirty = true;
        }
        this.pager.delegate = this._delegate;
        if (!this.items && this._childrenCount > 0) {
            pager_common_1.selectedIndexProperty.coerce(this);
            this._updateScrollPosition();
        }
    };
    Pager.prototype.onUnloaded = function () {
        if (this.pager) {
            this.pager.delegate = null;
        }
        _super.prototype.onUnloaded.call(this);
    };
    Pager.prototype.disposeNativeView = function () {
        this._delegate = null;
        this._dataSource = null;
        this._layout = null;
        if (this._observableArrayInstance) {
            this._observableArrayInstance.off(observable_array_1.ObservableArray.changeEvent, this._observableArrayHandler);
            this._observableArrayInstance = null;
        }
        _super.prototype.disposeNativeView.call(this);
    };
    Pager.prototype[pager_common_1.indicatorProperty.setNative] = function (value) {
        this._setIndicator(value);
    };
    Pager.prototype[pager_common_1.disableSwipeProperty.setNative] = function (value) {
        if (this.pager) {
            this.pager.scrollEnabled = !(String(value) === 'true');
        }
        this._disableSwipe = String(value) === 'true';
    };
    Object.defineProperty(Pager.prototype, "disableAnimation", {
        get: function () {
            return this._disableAnimation;
        },
        set: function (value) {
            this._disableAnimation = value;
        },
        enumerable: true,
        configurable: true
    });
    Pager.prototype._removeContainer = function (cell, indexPath) {
        var view = cell.view;
        var args = {
            eventName: pager_common_1.ITEMDISPOSING,
            object: this,
            index: indexPath.row,
            android: undefined,
            ios: cell,
            view: view
        };
        this.notify(args);
        view = args.view;
        if (view && view.parent) {
            if (!(view.parent instanceof Pager)) {
                this._removeView(view.parent);
            }
            view.parent._removeView(view);
        }
        this._map.delete(cell);
    };
    Pager.prototype.measure = function (widthMeasureSpec, heightMeasureSpec) {
        var _this = this;
        var changed = this._setCurrentMeasureSpecs(widthMeasureSpec, heightMeasureSpec);
        _super.prototype.measure.call(this, widthMeasureSpec, heightMeasureSpec);
        if (changed) {
            dispatch_async(main_queue, function () {
                if (!_this.pager) {
                    return;
                }
                _this.pager.reloadData();
                _this._updateScrollPosition();
            });
        }
    };
    Pager.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        var _this = this;
        _super.prototype.onMeasure.call(this, widthMeasureSpec, heightMeasureSpec);
        this._map.forEach(function (childView, pagerCell) {
            view_1.View.measureChild(_this, childView, childView._currentWidthMeasureSpec, childView._currentHeightMeasureSpec);
        });
    };
    Pager.prototype.onLayout = function (left, top, right, bottom) {
        var _this = this;
        _super.prototype.onLayout.call(this, left, top, right, bottom);
        this.pager.frame = this.nativeView.bounds;
        if (this.indicatorView && this.indicatorView.intrinsicContentSize) {
            this.indicatorView.center = CGPointMake(this.nativeView.center.x, this.nativeView.bounds.size.height - this.indicatorView.intrinsicContentSize.height);
        }
        var size = this._getSize();
        this._map.forEach(function (childView, pagerCell) {
            var width = view_1.layout.toDevicePixels(size.width);
            var height = view_1.layout.toDevicePixels(size.height);
            view_1.View.layoutChild(_this, childView, 0, 0, width, height);
        });
    };
    Pager.prototype.requestLayout = function () {
        if (!this._preparingCell) {
            _super.prototype.requestLayout.call(this);
        }
    };
    Pager.prototype._setPadding = function (newPadding) {
        var padding = {
            top: this._layout.sectionInset.top,
            right: this._layout.sectionInset.right,
            bottom: this._layout.sectionInset.bottom,
            left: this._layout.sectionInset.left
        };
        var newValue = Object.assign(padding, newPadding);
        this._layout.sectionInset = UIEdgeInsetsFromString("{" + newValue.top + "," + newValue.left + "," + newValue.bottom + "," + newValue.right + "}");
    };
    Pager.prototype._prepareCell = function (cell, indexPath) {
        try {
            this._preparingCell = true;
            var view = cell.view;
            var template = this._getItemTemplate(indexPath.row);
            if (!view) {
                view = template.createView();
            }
            var args = {
                eventName: pager_common_1.ITEMLOADING,
                object: this,
                index: indexPath.row,
                android: undefined,
                ios: cell,
                view: view
            };
            this.notify(args);
            view = args.view || this._getDefaultItemContent(indexPath.row);
            if (view instanceof proxy_view_container_1.ProxyViewContainer) {
                var sp = new stack_layout_1.StackLayout();
                sp.addChild(view);
                view = sp;
            }
            if (!cell.view) {
                cell.owner = new WeakRef(view);
            }
            else if (cell.view !== view) {
                this._map.delete(cell);
                this._removeContainer(cell, indexPath);
                cell.view.ios.removeFromSuperview();
                cell.owner = new WeakRef(view);
            }
            this._prepareItem(view, indexPath.row);
            this._map.set(cell, view);
            if (view && !view.parent) {
                this._addView(view);
                cell.contentView.addSubview(view.ios);
            }
            this._layoutCell(view, indexPath);
        }
        finally {
            this._preparingCell = false;
        }
    };
    Pager.prototype._layoutCell = function (cellView, index) {
        if (cellView) {
            var size = this._getSize();
            var width = view_1.layout.toDevicePixels(size.width);
            var height = view_1.layout.toDevicePixels(size.height);
            var widthMeasureSpec = view_1.layout.makeMeasureSpec(width, view_1.layout.EXACTLY);
            var heightMeasureSpec = view_1.layout.makeMeasureSpec(height, view_1.layout.EXACTLY);
            var measured = view_1.View.measureChild(this, cellView, widthMeasureSpec, heightMeasureSpec);
        }
    };
    Pager.prototype._addChildFromBuilder = function (name, value) {
        if (value instanceof common.PagerItem) {
            if (!this._childrenViews) {
                this._childrenViews = new Map();
            }
            var count = this._childrenViews.size;
            var keys = Array.from(this._childrenViews.keys());
            if (count === 0) {
                this._childrenViews.set(this._childrenCount, value);
            }
            else {
                for (var i = 0; i < count; i++) {
                    var key = keys[i];
                    var view = this._childrenViews.get(key);
                    if (i === keys.length - 1 && value !== view) {
                        this._childrenViews.set(this._childrenCount, value);
                    }
                }
            }
        }
    };
    Object.defineProperty(Pager.prototype, "horizontalOffset", {
        get: function () {
            return this.pager ? this.pager.contentOffset.x : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pager.prototype, "verticalOffset", {
        get: function () {
            return this.pager ? this.pager.contentOffset.y : 0;
        },
        enumerable: true,
        configurable: true
    });
    Pager.prototype._getSpacing = function () {
        return view_1.layout.toDeviceIndependentPixels(this.convertToSize(this.spacing));
    };
    Pager.prototype._getPeaking = function () {
        return view_1.layout.toDeviceIndependentPixels(this.convertToSize(this.peaking));
    };
    Pager.prototype._getSize = function (w, h) {
        if (w === void 0) { w = 0; }
        if (h === void 0) { h = 0; }
        var width = 0;
        var height = 0;
        if (width === 0) {
            width = view_1.layout.toDeviceIndependentPixels(this._effectiveItemWidth);
        }
        if (height === 0) {
            height = view_1.layout.toDeviceIndependentPixels(this._effectiveItemHeight);
        }
        if (this.orientation === 'vertical') {
            height = (height - ((this._getSpacing() * 2) + (this._getPeaking() * 2))) / this.perPage;
        }
        else {
            width = (width - ((this._getSpacing() * 2) + (this._getPeaking() * 2))) / this.perPage;
        }
        if (Number.isNaN(width)) {
            width = 0;
        }
        if (Number.isNaN(height)) {
            height = 0;
        }
        return { width: width, height: height };
    };
    __decorate([
        profiling_1.profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], Pager.prototype, "onLoaded", null);
    return Pager;
}(pager_common_1.PagerBase));
exports.Pager = Pager;
var PagerCell = (function (_super) {
    __extends(PagerCell, _super);
    function PagerCell() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(PagerCell.prototype, "view", {
        get: function () {
            return this.owner ? this.owner.get() : null;
        },
        enumerable: true,
        configurable: true
    });
    PagerCell.initWithEmptyBackground = function () {
        var cell = PagerCell.new();
        cell.backgroundColor = null;
        return cell;
    };
    PagerCell.prototype.willMoveToSuperview = function (newSuperview) {
        var parent = (this.view ? this.view.parent : null);
        if (parent && !newSuperview) {
            parent._removeContainer(this, this.index);
        }
    };
    return PagerCell;
}(UICollectionViewCell));
exports.PagerCell = PagerCell;
var UICollectionDelegateImpl = (function (_super) {
    __extends(UICollectionDelegateImpl, _super);
    function UICollectionDelegateImpl() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.currentPage = 0;
        _this.indexOfCellBeforeDragging = 0;
        return _this;
    }
    UICollectionDelegateImpl_1 = UICollectionDelegateImpl;
    UICollectionDelegateImpl.initWithOwner = function (owner) {
        var delegate = UICollectionDelegateImpl_1.alloc().init();
        delegate._owner = owner;
        delegate.startingScrollingOffset = CGPointMake(0, 0);
        return delegate;
    };
    UICollectionDelegateImpl.prototype.collectionViewLayoutInsetForSectionAtIndex = function (collectionView, collectionViewLayout, section) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            var inset = owner._getSpacing() + owner._getPeaking();
            if (owner.orientation === 'vertical') {
                return new UIEdgeInsets({ bottom: inset, left: 0, right: 0, top: inset });
            }
            return new UIEdgeInsets({ bottom: 0, left: inset, right: inset, top: 0 });
        }
        return new UIEdgeInsets({ bottom: 0, left: 0, right: 0, top: 0 });
    };
    UICollectionDelegateImpl.prototype.collectionViewLayoutSizeForItemAtIndexPath = function (collectionView, collectionViewLayout, indexPath) {
        var owner = this._owner ? this._owner.get() : null;
        if (!owner)
            return CGSizeZero;
        var size = owner._getSize();
        return CGSizeMake(size.width, size.height);
    };
    UICollectionDelegateImpl.prototype.collectionViewWillDisplayCellForItemAtIndexPath = function (collectionView, cell, indexPath) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            if (owner.items && indexPath.row === owner.items.length - owner.loadMoreCount) {
                owner.notify({
                    eventName: pager_common_1.LOADMOREITEMS,
                    object: owner
                });
            }
        }
        if (cell.preservesSuperviewLayoutMargins) {
            cell.preservesSuperviewLayoutMargins = false;
        }
        if (cell.layoutMargins) {
            cell.layoutMargins = UIEdgeInsetsZero;
        }
    };
    UICollectionDelegateImpl.prototype.collectionViewLayoutMinimumLineSpacingForSectionAtIndex = function (collectionView, collectionViewLayout, section) {
        var owner = this._owner ? this._owner.get() : null;
        if (!owner)
            return 0;
        return owner._getSpacing();
    };
    UICollectionDelegateImpl.prototype.scrollViewWillBeginDragging = function (scrollView) {
        this.startingScrollingOffset = scrollView.contentOffset;
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            if (owner.lastEvent === 0) {
                owner.notify({
                    eventName: Pager.swipeStartEvent,
                    object: owner
                });
                owner.lastEvent = 1;
            }
            this.indexOfCellBeforeDragging = owner.selectedIndex;
        }
    };
    UICollectionDelegateImpl.prototype.scrollViewDidEndScrollingAnimation = function (scrollView) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.notify({
                eventName: Pager.swipeEvent,
                object: owner
            });
            if (owner.indicatorView) {
                owner.indicatorView.progress = owner.selectedIndex;
            }
        }
    };
    UICollectionDelegateImpl.prototype.scrollViewDidScroll = function (scrollView) {
        var owner = this._owner.get();
        if (owner) {
            var width = 0;
            var w = (view_1.layout.toDeviceIndependentPixels(owner._effectiveItemWidth) - (((owner.perPage * 2) * owner._getSpacing()) + (owner._getPeaking() * 2))) / owner.perPage;
            var h = (view_1.layout.toDeviceIndependentPixels(owner._effectiveItemHeight) - (((owner.perPage * 2) * owner._getSpacing()) + (owner._getPeaking() * 2))) / owner.perPage;
            width = scrollView.contentOffset.x / w;
            if (owner.indicatorView && owner.indicatorView.setWithProgressAnimated) {
                owner.indicatorView.setWithProgressAnimated(width, true);
            }
            owner.notify({
                object: owner,
                eventName: Pager.scrollEvent,
                scrollX: owner.horizontalOffset,
                scrollY: owner.verticalOffset
            });
            if (owner.lastEvent === 1) {
                owner.notify({
                    eventName: Pager.swipeOverEvent,
                    object: owner
                });
                owner.lastEvent = 1;
            }
        }
    };
    UICollectionDelegateImpl.prototype.scrollViewWillEndDraggingWithVelocityTargetContentOffset = function (scrollView, velocity, targetContentOffset) {
        var owner = this._owner ? this._owner.get() : null;
        if (!owner)
            return;
        if (owner.lastEvent === 1) {
            owner.notify({
                eventName: Pager.swipeEndEvent,
                object: owner
            });
            owner.lastEvent = 0;
        }
        var collection = owner.pager;
        var contentSize = owner.orientation === 'vertical' ? collection.contentSize.height : collection.contentSize.width;
        if (contentSize === 0) {
            return;
        }
        var target = targetContentOffset.value;
        var currentVelocity = 0;
        if (owner.orientation === 'vertical') {
            currentVelocity = velocity.y;
        }
        else {
            currentVelocity = velocity.x;
        }
        var next = 0;
        if (currentVelocity > 0.5) {
            next = 1;
        }
        else if (currentVelocity < -0.5) {
            next = -1;
        }
        var v = owner.orientation === 'vertical' ? target.y - this.startingScrollingOffset.y : target.x - this.startingScrollingOffset.x;
        var w = (view_1.layout.toDeviceIndependentPixels(owner._effectiveItemWidth) - (((owner.perPage * 2) * owner._getSpacing()) + (owner._getPeaking() * 2))) / owner.perPage;
        var h = (view_1.layout.toDeviceIndependentPixels(owner._effectiveItemHeight) - (((owner.perPage * 2) * owner._getSpacing()) + (owner._getPeaking() * 2))) / owner.perPage;
        var center = (owner.orientation === 'vertical' ? h / 2 : w / 2);
        if (next === 0) {
            if (v < 0) {
                if (v <= -center) {
                    next = -1;
                }
            }
            else if (v > 0) {
                if (v >= center) {
                    next = 1;
                }
            }
        }
        if (next > 1) {
            next = 1;
        }
        else if (next < -1) {
            next = -1;
        }
        var nextIndex = this.indexOfCellBeforeDragging + next;
        if (nextIndex === -1) {
            nextIndex = 0;
        }
        else if (nextIndex > (owner._childrenCount - 1)) {
            next = (owner._childrenCount - 1);
        }
        var attribute = collection.collectionViewLayout.layoutAttributesForItemAtIndexPath(NSIndexPath.indexPathForRowInSection(nextIndex, 0));
        var x = owner.orientation === 'vertical' ? target.x : attribute.frame.origin.x - (owner._getSpacing() + owner._getPeaking());
        var y = owner.orientation === 'vertical' ? attribute.frame.origin.y - (owner._getSpacing() + owner._getPeaking()) : target.y;
        targetContentOffset.value = CGPointMake(x, y);
        pager_common_1.selectedIndexProperty.nativeValueChange(owner, nextIndex);
    };
    var UICollectionDelegateImpl_1;
    UICollectionDelegateImpl = UICollectionDelegateImpl_1 = __decorate([
        ObjCClass(UICollectionViewDelegate, UICollectionViewDelegateFlowLayout)
    ], UICollectionDelegateImpl);
    return UICollectionDelegateImpl;
}(NSObject));
var UICollectionViewDataSourceImpl = (function (_super) {
    __extends(UICollectionViewDataSourceImpl, _super);
    function UICollectionViewDataSourceImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UICollectionViewDataSourceImpl_1 = UICollectionViewDataSourceImpl;
    UICollectionViewDataSourceImpl.initWithOwner = function (owner) {
        var delegate = UICollectionViewDataSourceImpl_1.alloc().init();
        delegate._owner = owner;
        return delegate;
    };
    UICollectionViewDataSourceImpl.prototype.collectionViewCellForItemAtIndexPath = function (collectionView, indexPath) {
        var owner = this._owner ? this._owner.get() : null;
        var cell;
        if (owner && !owner.items && owner._childrenCount > 0) {
            owner._preparingCell = true;
            var size = owner._getSize();
            collectionView.registerClassForCellWithReuseIdentifier(PagerCell.class(), "static-" + indexPath.row);
            cell =
                collectionView.dequeueReusableCellWithReuseIdentifierForIndexPath("static-" + indexPath.row, indexPath) || PagerCell.initWithEmptyBackground();
            cell.index = indexPath;
            var view = owner._childrenViews.get(indexPath.row);
            if (view instanceof proxy_view_container_1.ProxyViewContainer) {
                var sp = new stack_layout_1.StackLayout();
                sp.addChild(view);
                view = sp;
            }
            if (!cell.view) {
                cell.owner = new WeakRef(view);
            }
            else if (cell.view !== view) {
                cell.view.ios.removeFromSuperview();
                cell.owner = new WeakRef(view);
            }
            if (view && !view.parent) {
                owner._addView(view);
                cell.contentView.addSubview(view.ios);
            }
            else if (view && view.ios) {
                cell.contentView.addSubview(view.ios);
            }
            owner._layoutCell(view, indexPath);
            var width = view_1.layout.toDevicePixels(size.width);
            var height = view_1.layout.toDevicePixels(size.height);
            if (view && view.isLayoutRequired) {
                view_1.View.layoutChild(owner, view, 0, 0, width, height);
            }
            owner._preparingCell = false;
            return cell;
        }
        var template = owner && owner._getItemTemplate(indexPath.row);
        cell =
            collectionView.dequeueReusableCellWithReuseIdentifierForIndexPath(template.key, indexPath) || PagerCell.initWithEmptyBackground();
        cell.index = indexPath;
        if (owner) {
            var size = owner._getSize();
            owner._prepareCell(cell, indexPath);
            var cellView = cell.view;
            if (cellView && cellView.isLayoutRequired) {
                view_1.View.layoutChild(owner, cellView, 0, 0, view_1.layout.toDevicePixels(size.width), view_1.layout.toDevicePixels(size.height));
            }
        }
        return cell;
    };
    UICollectionViewDataSourceImpl.prototype.collectionViewNumberOfItemsInSection = function (collectionView, section) {
        var owner = this._owner ? this._owner.get() : null;
        if (!owner)
            return 0;
        return owner._childrenCount;
    };
    UICollectionViewDataSourceImpl.prototype.numberOfSectionsInCollectionView = function (collectionView) {
        return 1;
    };
    var UICollectionViewDataSourceImpl_1;
    UICollectionViewDataSourceImpl = UICollectionViewDataSourceImpl_1 = __decorate([
        ObjCClass(UICollectionViewDataSource)
    ], UICollectionViewDataSourceImpl);
    return UICollectionViewDataSourceImpl;
}(NSObject));
var UICollectionViewFlowLinearLayoutImpl = (function (_super) {
    __extends(UICollectionViewFlowLinearLayoutImpl, _super);
    function UICollectionViewFlowLinearLayoutImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UICollectionViewFlowLinearLayoutImpl.initWithOwner = function (owner) {
        var layout = UICollectionViewFlowLinearLayoutImpl.alloc().init();
        layout._owner = owner;
        return layout;
    };
    UICollectionViewFlowLinearLayoutImpl.prototype.layoutAttributesForElementsInRect = function (rect) {
        var owner = this._owner ? this._owner.get() : null;
        var originalLayoutAttribute = _super.prototype.layoutAttributesForElementsInRect.call(this, rect);
        var visibleLayoutAttributes = [];
        if (owner) {
            if (owner.transformers && owner.transformers.indexOf('scale') > -1) {
                var count = originalLayoutAttribute.count;
                for (var i = 0; i < count; i++) {
                    var attributes = originalLayoutAttribute.objectAtIndex(i);
                    visibleLayoutAttributes[i] = attributes;
                    var frame = attributes.frame;
                    var width = attributes.frame.size.width * .75;
                    var height = attributes.frame.size.height * .75;
                    attributes.frame.size.width = width;
                    attributes.frame.size.height = height;
                    var spacing = owner.convertToSize(owner.spacing);
                    var distance = Math.abs(this.collectionView.contentOffset.x + this.collectionView.contentInset.left + spacing - frame.origin.x);
                    var scale = Math.min(Math.max(1 - distance / (this.collectionView.bounds.size.width), .75), 1);
                    attributes.transform = CGAffineTransformScale(attributes.transform, 1, scale);
                }
            }
            else {
                return originalLayoutAttribute;
            }
        }
        return visibleLayoutAttributes;
    };
    UICollectionViewFlowLinearLayoutImpl.prototype.shouldInvalidateLayoutForBoundsChange = function (newBounds) {
        return true;
    };
    UICollectionViewFlowLinearLayoutImpl.prototype.initialLayoutAttributesForAppearingItemAtIndexPath = function (itemIndexPath) {
        var attrs = _super.prototype.initialLayoutAttributesForAppearingItemAtIndexPath.call(this, itemIndexPath);
        attrs.alpha = 1;
        return attrs;
    };
    UICollectionViewFlowLinearLayoutImpl.prototype.finalLayoutAttributesForDisappearingItemAtIndexPath = function (itemIndexPath) {
        var attrs = _super.prototype.finalLayoutAttributesForDisappearingItemAtIndexPath.call(this, itemIndexPath);
        attrs.alpha = 1;
        return attrs;
    };
    return UICollectionViewFlowLinearLayoutImpl;
}(UICollectionViewFlowLayout));
//# sourceMappingURL=pager.ios.js.map