"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var pager_1 = require("../pager");
var ReactNativeScript = require("react-nativescript");
var ContainerView_1 = require("react-nativescript/dist/components/ContainerView");
var EventHandling_1 = require("react-nativescript/dist/client/EventHandling");
var ElementRegistry_1 = require("react-nativescript/dist/client/ElementRegistry");
ElementRegistry_1.register('pager', pager_1.Pager);
ElementRegistry_1.register('pagerItem', pager_1.PagerItem);
var _Pager = (function (_super) {
    __extends(_Pager, _super);
    function _Pager(props) {
        var _this = _super.call(this, props) || this;
        _this.argsViewToRootKeyAndRef = new Map();
        _this.roots = new Set();
        _this.defaultOnItemLoading = function (args) {
            var _a = _this.props._debug, logLevel = _a.logLevel, onCellRecycle = _a.onCellRecycle, onCellFirstLoad = _a.onCellFirstLoad;
            var _b = _this.props, items = _b.items, itemTemplateSelector = _b.itemTemplateSelector;
            var item = _Pager.isItemsSource(items) ? items.getItem(args.index) : items[args.index];
            var template = itemTemplateSelector
                ? typeof itemTemplateSelector === 'string'
                    ? itemTemplateSelector
                    : itemTemplateSelector(item, args.index, items)
                : null;
            var cellFactory = template === null
                ? _this.props.cellFactory
                : _this.props.cellFactories
                    ? _this.props.cellFactories.get(template).cellFactory
                    : _this.props.cellFactory;
            if (typeof cellFactory === 'undefined') {
                console.warn("Pager: No cell factory found, given template " + template + "!");
                return;
            }
            var view = args.view;
            if (!view) {
                var rootKeyAndRef = _this.renderNewRoot(item, cellFactory);
                args.view = rootKeyAndRef.ref.current;
                _this.argsViewToRootKeyAndRef.set(args.view, rootKeyAndRef);
                if (onCellFirstLoad)
                    onCellFirstLoad(rootKeyAndRef.ref.current);
            }
            else {
                console.log("[Pager] existing view: ", view);
                if (onCellRecycle)
                    onCellRecycle(view);
                var data = _this.argsViewToRootKeyAndRef.get(view);
                var rootKey = void 0;
                var ref = void 0;
                if (data) {
                    rootKey = data.rootKey;
                    ref = data.ref;
                }
                else {
                    var rootKeyAndRef = _this.renderNewRoot(item, cellFactory);
                    rootKey = rootKeyAndRef.rootKey;
                    ref = rootKeyAndRef.ref;
                }
                if (typeof rootKey === 'undefined') {
                    console.error("Unable to find root key that args.view corresponds to!", view);
                    return;
                }
                if (typeof ref === 'undefined') {
                    console.error("Unable to find ref that args.view corresponds to!", view);
                    return;
                }
                ReactNativeScript.render(cellFactory(item, ref), null, function () {
                }, rootKey);
            }
        };
        _this.renderNewRoot = function (item, cellFactory) {
            console.log("[Pager] no existing view.");
            var ref = React.createRef();
            var rootKey = 'Pager-' + _this.roots.size.toString();
            ReactNativeScript.render(cellFactory(item, ref), null, function () {
            }, rootKey);
            _this.roots.add(rootKey);
            return {
                rootKey: rootKey,
                ref: ref,
            };
        };
        _this.state = {
            nativeCells: {},
            nativeCellToItemIndex: new Map(),
            itemIndexToNativeCell: props._debug.logLevel === 'debug' ? new Map() : undefined,
        };
        return _this;
    }
    _Pager.prototype.updateListeners = function (node, attach, nextProps) {
        _super.prototype.updateListeners.call(this, node, attach, nextProps);
        if (attach === null) {
            EventHandling_1.updateListener(node, pager_1.Pager.loadMoreItemsEvent, this.props.onLoadMoreItems, nextProps.onLoadMoreItems);
        }
        else {
            var method = (attach ? node.on : node.off).bind(node);
            method(pager_1.Pager.itemLoadingEvent, this.defaultOnItemLoading);
            if (this.props.onLoadMoreItems)
                method(pager_1.Pager.loadMoreItemsEvent, this.props.onLoadMoreItems);
        }
    };
    _Pager.prototype.componentDidMount = function () {
        var _this = this;
        _super.prototype.componentDidMount.call(this);
        var node = this.getCurrentRef();
        if (!node) {
            console.warn("React ref to NativeScript View lost, so unable to set item templates.");
            return;
        }
        if (this.props.cellFactories) {
            var itemTemplates_1 = [];
            this.props.cellFactories.forEach(function (info, key) {
                var placeholderItem = info.placeholderItem, cellFactory = info.cellFactory;
                itemTemplates_1.push({
                    key: key,
                    createView: function () {
                        console.log("[Pager] item template \"" + key + "\"");
                        var rootKeyAndRef = _this.renderNewRoot(placeholderItem, cellFactory);
                        _this.argsViewToRootKeyAndRef.set(rootKeyAndRef.ref.current, rootKeyAndRef);
                        return rootKeyAndRef.ref.current;
                    },
                });
            });
            node.itemTemplates = itemTemplates_1;
        }
    };
    _Pager.prototype.componentWillUnmount = function () {
        _super.prototype.componentWillUnmount.call(this);
        this.roots.forEach(function (root) { return ReactNativeScript.unmountComponentAtNode(root); });
    };
    _Pager.isItemsSource = function (arr) {
        return typeof arr.getItem === 'function';
    };
    _Pager.prototype.render = function () {
        console.log("Pager's render()");
        var _a = this.props, forwardedRef = _a.forwardedRef, onLoaded = _a.onLoaded, onUnloaded = _a.onUnloaded, onAndroidBackPressed = _a.onAndroidBackPressed, onShowingModally = _a.onShowingModally, onShownModally = _a.onShownModally, onTap = _a.onTap, onDoubleTap = _a.onDoubleTap, onPinch = _a.onPinch, onPan = _a.onPan, onSwipe = _a.onSwipe, onRotation = _a.onRotation, onLongPress = _a.onLongPress, onTouch = _a.onTouch, onPropertyChange = _a.onPropertyChange, children = _a.children, _debug = _a._debug, items = _a.items, rest = __rest(_a, ["forwardedRef", "onLoaded", "onUnloaded", "onAndroidBackPressed", "onShowingModally", "onShownModally", "onTap", "onDoubleTap", "onPinch", "onPan", "onSwipe", "onRotation", "onLongPress", "onTouch", "onPropertyChange", "children", "_debug", "items"]);
        return React.createElement('pager', __assign({ className: 'pager-group' }, rest, { items: items, ref: forwardedRef || this.myRef }), children);
    };
    _Pager.defaultProps = {
        _debug: {
            logLevel: 'info',
            onCellFirstLoad: undefined,
            onCellRecycle: undefined,
        },
    };
    return _Pager;
}(ContainerView_1.RCTContainerView));
exports._Pager = _Pager;
var ɵ0 = function (props, ref) {
    var children = props.children, rest = __rest(props, ["children"]);
    return React.createElement(_Pager, __assign({}, rest, { forwardedRef: ref }), children);
};
exports.ɵ0 = ɵ0;
exports.$Pager = React.forwardRef(ɵ0);
__export(require("./pager-item"));
//# sourceMappingURL=index.js.map